// ignore_for_file: use_build_context_synchronously

import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:mobile_pos/mobile_pos_sdk.dart';
import 'package:mobile_pos/src/helpers/global_variables.dart';
import 'package:mobile_pos/src/helpers/helper_functions.dart';
import 'package:mobile_pos/src/helpers/logger.dart';
import 'package:mobile_pos/src/helpers/urls.dart';
import 'package:mobile_pos/src/models/alert_number_model.dart';
import 'package:mobile_pos/src/models/bankbox_transaction_model.dart';
import 'package:mobile_pos/src/models/card_tx_response.dart';
import 'package:mobile_pos/src/models/pick_up_location.dart';
import 'package:mobile_pos/src/models/raven_const.dart';
import 'package:mobile_pos/src/models/success_response.dart';
import 'package:mobile_pos/src/models/terminal_model.dart';
import 'package:mobile_pos/src/network/http_base.dart';
import 'package:mobile_pos/src/shared_widgets/progress_dialog.dart';
import 'package:mobile_pos/src/views/card_payment/card_balance.dart';
import 'package:mobile_pos/src/views/card_payment/card_failure_page.dart';
import 'package:mobile_pos/src/views/card_payment/card_success_page_new.dart';

class ApiRequests {
  static Future<bool> sendWhatsappReceipt(context, phone, rrn,
      {String? category}) async {
    final payload = {
      "phone_number": phone.toString(),
      "rrn": rrn.toString(),
    };

    showDialog(
        barrierDismissible: false,
        context: context,
        builder: (BuildContext context) => const ProgressDialog(
              status: 'Processing...',
            ));

    var response = await HttpBase.postRequest(
        payload,
        isFullRoute: category != null,
        category == null
            ? 'pdon/whatsapp_receipt'
            : URLS.getWhatsappReceiptUrl(category));
    Navigator.pop(context);

    if (response == null || response == 'failed') {
      showSnack(context, 'Unable to send receipt at the moment');
      return false;
    }

    if (response['status'] == 'success') {
      showSnack(context, 'Receipt was sent successfully');
      return true;
    }
    if (response['status'] == 'fail') {
      showSnack(context, response['message']);
      return false;
    }

    return false;
  }

  static Future<bool> sendEmailReceipt(context, email, rrn,
      {String? category}) async {
    final payload = {
      "destination": "email",
      "receiverEmail": email.toString(),
      "rrn": rrn.toString(),
    };

    showDialog(
        barrierDismissible: false,
        context: context,
        builder: (BuildContext context) => const ProgressDialog(
              status: 'Processing...',
            ));

    var response = await HttpBase.postRequest(
        payload,
        isFullRoute: category != null,
        category == null
            ? 'pdon/share_receipt'
            : URLS.getEmailReceiptUrl(category));
    Navigator.pop(context);

    if (response == null || response == 'failed') {
      showSnack(context, 'Unable to send receipt at the moment');
      return false;
    }

    if (response['status'] == 'success') {
      showSnack(context, 'Receipt was sent successfully');
      return true;
    }

    if (response['status'] == 'fail') {
      showSnack(context, response['message']);
      return false;
    }

    return false;
  }

  static Future<dynamic> getQrCodeLink(context, rrn, {String? category}) async {
    showDialog(
        barrierDismissible: false,
        context: context,
        builder: (BuildContext context) => const ProgressDialog(
              status: 'Processing...',
            ));
    var response = await HttpBase.getRequestWithHeader(
        {},
        category == null
            ? 'pdon/get/qrcode_receipt?rrn=$rrn'
            : URLS.getQrCodeReceiptUrl(category, rrn),
        isFullRoute: category != null);

    Navigator.pop(context);

    if (response == null && response.toString() == 'failed') {
      showSnack(context, 'Unable to generate QR Code');
      return false;
    }

    if (response['status'] == 'success') {
      showSnack(context, 'Qr Code generated successfully');
      return response['data'];
    }

    return false;
  }

  static Future<void> getCardBalance(
      BuildContext context, String cardPayload) async {
    final cardModel =
        CardTransactionResponseModel.fromJson(jsonDecode(cardPayload));
    Map<String, String> headerMap = {
      'x-device-model': 'K11',
      'x-brand': 'Horizonpay',
      'x-app-version': "1.0.0",
      'x-serial-no': affiliateInfo?.serialNumber ?? "98230303973867",
      "Content-Type": "application/json"
    };
    showDialog(
        barrierDismissible: false,
        context: context,
        builder: (BuildContext context) => const ProgressDialog(
              status: 'Processing...',
            ));
    Map<String, dynamic> nibssMap = cardModel.nibbsEMV!.toJson();
    nibssMap.update('ssl', (value) => 'true');

    var field52 = cardModel.nibbsEMV!.field52;
    if (!nibssMap.containsKey("field52")) {
      nibssMap.putIfAbsent('field52', () => '');
    } else {
      if (field52 == null) {
        nibssMap.update('field52', (value) => '');
      }

      if (field52 == 'null') {
        nibssMap.update('field52', (value) => '');
      }
    }

    var pinBlock = cardModel.nibbsEMV!.pinblock;
    if (!nibssMap.containsKey("pinblock")) {
      nibssMap.putIfAbsent('pinblock', () => '');
    } else {
      if (pinBlock == null) {
        nibssMap.update('pinblock', (value) => '');
      }

      if (pinBlock == 'null') {
        nibssMap.update('pinblock', (value) => '');
      }
    }
    final serializedMap = nibssMap.map((key, value) {
      return MapEntry(key, value);
    });

    var response = await HttpBase.postRequestJson(
        serializedMap, "http://46.101.123.218:5010/api/v1/card/checkBalance",
        isFullRoute: true, header: headerMap);
    Navigator.pop(context);
    try {
      if (response['status'] && response['resp'] == "00") {
        replaceRoute(
            context,
            CardBalance(
              maskedPan: cardModel.maskedPAN,
              balance: (double.tryParse(response['balance']) ?? 0.0) / 100,
            ));
        return;
      } else {
        replaceRoute(
            context,
            CardFailurePage(
              paymentType: PaymentType.checkBalance,
              reason: response['meaning'] ??
                  "something went wrong... please try again.",
            ));
        return;
      }
    } catch (ex) {
      replaceRoute(
          context,
          const CardFailurePage(
            paymentType: PaymentType.checkBalance,
            reason: "something went wrong... please try again.",
          ));
    }
  }

  static Future<void> processCard(
      BuildContext context, double amount, String cardData) async {
    final body = json.decode(cardData);

    final cardModel = CardTransactionResponseModel.fromJson(body);

    showDialog(
        barrierDismissible: false,
        context: context,
        builder: (BuildContext context) => const ProgressDialog(
              status: 'Processing...',
            ));

    Map<String, dynamic> nibssMap = cardModel.nibbsEMV!.toJson();
    nibssMap.update('ssl', (value) => 'true');
    var field52 = cardModel.nibbsEMV!.field52;
    if (!nibssMap.containsKey("field52")) {
      nibssMap.putIfAbsent('field52', () => '');
    } else {
      if (field52 == null) {
        nibssMap.update('field52', (value) => '');
      }

      if (field52 == 'null') {
        nibssMap.update('field52', (value) => '');
      }
    }

    var pinBlock = cardModel.nibbsEMV!.pinblock;
    if (!nibssMap.containsKey("pinblock")) {
      nibssMap.putIfAbsent('pinblock', () => '');
    } else {
      if (pinBlock == null) {
        nibssMap.update('pinblock', (value) => '');
      }

      if (pinBlock == 'null') {
        nibssMap.update('pinblock', (value) => '');
      }
    }

    final payload = <String, dynamic>{};

    final serializedMap = nibssMap.map((key, value) {
      return MapEntry(key, value);
    });
    if (serializedMap['processor'] == "") {
      serializedMap.remove("processor");
    }

    payload['card_data'] = serializedMap;
    payload['poseidon_serial_number'] = cardModel.ravenEMV!.pSerialNo ?? "";
    payload['card_method'] = cardModel.ravenEMV!.cardMethod;
    payload['app_profile'] = {
      "affiliate_app_name": pluginConfig.appInfo.appName,
      "affiliate_app_id": pluginConfig.appInfo.appId,
      "affiliate_app_email": pluginConfig.appInfo.appEmail,
      "bvn": pluginConfig.customerInfo.bvn
    };

    var response = await HttpBase.postRequestJson(
        payload, URLS.cardProcessingUrl(bluetoothModel?.category ?? ''),
        isFullRoute: true);
    Navigator.pop(context);

    if (response == 'failed' || response == null) {
      final error = BankBoxException(
          code: kNibbsError, message: 'Transaction failed to complete');
      replaceRoute(
          context,
          CardFailurePage(
            reason: error.message,
          ));
      pluginConfig.onError.call(error);
    } else {
      if (response['data']['data']['resp'].toString() == '00') {
        final res = BankBoxResponse(data: Map.from(response));
        replaceRoute(
            context,
            CardSuccessPageNew(
              amount: amount,
              response: res,
              rrn: cardModel.nibbsEMV!.rrn,
              stan: cardModel.nibbsEMV!.stan ?? "",
              cardExpiry: cardModel.nibbsEMV!.expirydate ?? "",
              cardNumber: cardModel.nibbsEMV!.pan ?? "",
              terminalSerial: bluetoothModel?.serialNo ?? "",
            ));
        pluginConfig.onSuccess.call(res);
      } else {
        String? message;
        String? responseCode;

        try {
          message = refineMeaning(response);
          responseCode = response['data']['data']['resp'];
        } catch (e) {}

        //Unsuccessful
        final error = BankBoxException(
            code: kNibbsError,
            message:
                'Reason${message != null ? ': $responseCode / ${message.toLowerCase()}' : ''}');

        replaceRoute(
            context,
            CardFailurePage(
              reason: error.message,
              rrn: cardModel.nibbsEMV!.rrn,
            ));
        pluginConfig.onError.call(error);
      }
    }
  }

  static Future<void> fetchAppInfo() async {
    if (affiliateInfo != null) return;

    final payload = {"affiliate_app_id": pluginConfig.appInfo.appId};

    // print(jsonEncode(payload));

    var response =
        await HttpBase.postRequest(payload, 'pdon/affiliate_app_setting');

    if (response != null && response != 'failed') {
      if (response['data'] != null) {
        affiliateInfo =
            AffiliateInfoModel.fromJson(response['data']['appsetting']);
      }
    }
    logData(response);
  }

  static Future<bool> getKYBStatus() async {
    final payload = {
      "affiliate_app_id": pluginConfig.appInfo.appId,
      "email": pluginConfig.customerInfo.email
    };

    var response =
        await HttpBase.postRequest(payload, 'pdon/business_kyb_status');

    if (response != null && response != 'failed') {
      if (response['data'] != null) {
        return response['data'] != "completed";
      }
    }
    return true;
  }

  static Future<void> registerUser() async {
    final payload = pluginConfig.customerInfo.toJson();
    payload["affiliate_app_id"] = pluginConfig.appInfo.appId;
    //logData(jsonEncode(payload));
    await HttpBase.postRequest(payload, 'pdon/register');
    //print(response);
    //logData(response);
  }

  static Future<bool> updateUserDetails({
    required String businessName,
    required String businessType,
    required String businessNIN,
    required businessBVN,
    required String businessAddress,
    required String state,
    required String lga,
  }) async {
    final payload = {
      "email": pluginConfig.customerInfo.email,
      "affiliate_app_id": pluginConfig.appInfo.appId,
      "busines_name": businessName,
      "business_address": businessAddress,
      "business_description": "posideon product",
      "business_category": businessType,
      "phone": pluginConfig.customerInfo.phone,
      "state": state,
      "lga": lga,
      "lname": pluginConfig.customerInfo.lastName,
      "fname": pluginConfig.customerInfo.firstName,
      "nin": businessNIN,
      "bvn": businessBVN,
      "status": "1"
    };

    var response = await HttpBase.postRequest(payload, 'pdon/update');

    if (response == 'failed' || response == null) {
      final error = BankBoxException(
          code: kUpdateUserError, message: 'User account update failed');
      pluginConfig.onError(error);
      return false;
    } else {
      return response['status'] == 'success';
    }
  }

  static Future<bool> requestBankBox({
    required int qty,
    required String phone,
    String? state,
    String? lga,
    String? address,
    String? landmark,
    required DeliveryMethod deliveryMethod,
    required BuildContext context,
  }) async {
    final payload = {
      "quantity": qty.toString(),
      "phone": phone,
      "delivery_state": state,
      "delivery_address": address,
      "delivery_lga": lga,
      "landmark": landmark ?? 'pickup',
      "purpose": "Collect business payments",
      "delivery_option": deliveryMethod.name
    };
    payload["poseidon_email"] = pluginConfig.customerInfo.email;
    payload["affiliate_app_id"] = pluginConfig.appInfo.appId;
    var response = await HttpBase.postRequest(payload, 'pdon/request_bankbox');

    //print(response.toString());
    //logData(response);

    //failed
    if (response == 'failed' || response == null) {
      final error = BankBoxException(
          code: kRequestTerminalError, message: 'Terminal request failed');
      pluginConfig.onError(error);
      return false;
    } else {
      if (response['status'] == 'success') {
        return true;
      } else {
        showSnack(context,
            toBeginningOfSentenceCase(response['message'].toString()) ?? '');
        return false;
      }
    }
  }

  static Future<List<TerminalRequestModel>> getUserTerminalRequests() async {
    final payload = {
      "poseidon_email": pluginConfig.customerInfo.email,
      "affiliate_app_id": pluginConfig.appInfo.appId,
    };

    var response = await HttpBase.postRequest(payload, 'pdon/bankbox_request');
    //logData(response);

    //failed
    if (response == 'failed' || response == null) {
      final error = BankBoxException(
          code: kRequestTerminalError,
          message: 'Get user terminal requests failed');
      pluginConfig.onError(error);
      return [];
    } else {
      final list = (response['data'] as List)
          .map((e) => TerminalRequestModel.fromJson(e))
          .toList()
          .where((element) => element.status != 4)
          .toList();
      return list;
    }
  }

  static Future<List<TerminalModel>> getUserBankBox() async {
    final payload = {
      "poseidon_email": pluginConfig.customerInfo.email,
      "affiliate_app_id": pluginConfig.appInfo.appId,
    };

    var response = await HttpBase.postRequest(payload, 'pdon/user_bankbox');
    //logData(response);

    //failed
    if (response == 'failed' || response == null || response['data'] == null) {
      final error = BankBoxException(
          code: kRequestTerminalError, message: 'Get user terminals failed');
      pluginConfig.onError(error);
      return [];
    } else {
      final list = (response['data'] as List)
          .map((e) => TerminalModel.fromJson(e))
          .toList();
      return list;
    }
  }

  static Future<bool> updateBankBoxLable({
    required String label,
    required String serialNumber,
    String? nin,
  }) async {
    final payload = {"label": label, "serial_number": serialNumber};
    payload["affiliate_app_id"] = pluginConfig.appInfo.appId;
    var response = await HttpBase.postRequest(payload, 'pdon/update_bankbox');
    logData(response);

    //failed
    if (response == 'failed' || response == null) {
      final error = BankBoxException(
          code: kDeviceLabelUpdateError,
          message: 'Could not update device label');
      pluginConfig.onError(error);
      return false;
    } else {
      return response['status'] == 'success';
    }
  }

  static Future<dynamic> getBluetoothDeviceInfo({
    required String serialNumber,
  }) async {
    final payload = {"serial_no": serialNumber};
    var response = await HttpBase.postRequest(payload, URLS.getTerminalUrl(),
        isFullRoute: true);
    logData(response);
    if (response == 'failed' || response == null) {
      final error = BankBoxException(
          code: kDeviceLabelUpdateError,
          message: 'Could not update device label');
      pluginConfig.onError(error);
      return false;
    } else {
      if (response['status'] == "success") {
        return response['data'];
      }
      return false;
    }
  }

  static Future<dynamic> getNetworkStates() async {
    var response = await HttpBase.getRequestWithHeader(
        {},
        pluginConfig.isStaging
            ? 'https://postest.getravenpos.com/pdon/network_stability_summary'
            : 'https://posapi.getravenbank.com/v1/pdon/network_stability_summary',
        isFullRoute: true);

    if (response == 'failed' || response == null) {
      return false;
    } else {
      if (response['status'] == "success") {
        return response['data'];
      }
      return false;
    }
  }

  static Future<BankBoxTransactionModel?> getTerminalTransactions() async {
    var response = await HttpBase.getRequestWithHeader({},
        "pdon/transactions?affiliate_app_id=${affiliateInfo!.appId}&email=${pluginConfig.customerInfo.email}",
        isFullRoute: false);

    if (response == 'failed' || response == null) {
      return null;
    } else {
      if (response['status'] == "success") {
        return BankBoxTransactionModel.fromJson(response['data']);
      }
      return null;
    }
  }

  static Future<BankBoxTransactionModel?> getTransactionByRange(
      DateTime startDate, DateTime stopDate) async {
    var response = await HttpBase.getRequestWithHeader({},
        "pdon/transactions?affiliate_app_id=${affiliateInfo!.appId}&email=${pluginConfig.customerInfo.email}&start=${formateDateForTranx(startDate)}&end=${formateDateForTranx(stopDate)}",
        isFullRoute: false);

    if (response == 'failed' || response == null) {
      return null;
    } else {
      if (response['status'] == "success") {
        return BankBoxTransactionModel.fromJson(response['data']);
      }
      return null;
    }
  }

  static Future<dynamic> logDispute({
    required String transactionId,
    required String customerName,
    required String customerPhone,
    required disputeType,
    required String description,
    required String bank,
    required String accountNumber,
    required String amount,
  }) async {
    final payload = {
      "issuing_app_id": affiliateInfo!.appId,
      "type": disputeType,
      "bank_name": bank,
      "customer_name": customerName,
      "customer_phone": customerPhone,
      "note": description,
      "transaction_id": transactionId,
      "amount": amount,
      "account_number": accountNumber,
      "email": pluginConfig.customerInfo.email
    };

    var response = await HttpBase.postRequest(payload, 'pdon/log_dispute',
        returnErrorMsg: true);

    if (response == 'failed' || response == null) {
      final error = BankBoxException(
          code: kUpdateUserError, message: 'User account update failed');
      pluginConfig.onError(error);
      return false;
    } else {
      return response;
    }
  }

  static Future<dynamic> addAlertNumber(
      {required String phoneNumber,
      required String tid,
      required serial,
      required firstname,
      required lastname,
      required String email,
      required bool whatsappEnabled,
      required bool emailEnabled}) async {
    final payload = {
      "poseidon_owner_email": pluginConfig.appInfo.appEmail,
      "issuing_app_id": pluginConfig.appInfo.appId,
      "tid": tid,
      "serial": serial,
      "phone": phoneNumber,
      "fname": firstname,
      "lname": lastname,
      "contact_email": email,
      "whatsapp_enabled": whatsappEnabled ? "1" : "0",
      "sms_enabled": "0",
      "email_enabled": emailEnabled ? "1" : "0",
    };

    var response = await HttpBase.postRequest(
        payload, 'pdon/alert/add_phone_number',
        returnErrorMsg: true);

    if (response == 'failed' || response == null) {
      final error = BankBoxException(
          code: kUpdateUserError, message: 'User account update failed');
      pluginConfig.onError(error);
      return false;
    } else {
      return response;
    }
  }

  static Future<dynamic> updateAlertNumber(
      {required String id,
      required String phoneNumber,
      required String tid,
      required serial,
      required firstname,
      required lastname,
      required String email,
      required bool whatsappEnabled,
      required bool emailEnabled}) async {
    final payload = {
      "poseidon_owner_email": pluginConfig.appInfo.appEmail,
      "issuing_app_id": pluginConfig.appInfo.appId,
      "tid": tid,
      "id": id,
      "serial": serial,
      "phone": phoneNumber,
      "fname": firstname,
      "lname": lastname,
      "contact_email": email,
      "whatsapp_enabled": whatsappEnabled ? "1" : "0",
      "sms_enabled": "0",
      "email_enabled": emailEnabled ? "1" : "0",
    };

    var response = await HttpBase.postRequest(
        payload, 'pdon/alert/update_alert_number',
        returnErrorMsg: true);

    if (response == 'failed' || response == null) {
      final error = BankBoxException(
          code: kUpdateUserError, message: 'User account update failed');
      pluginConfig.onError(error);
      return false;
    } else {
      return response;
    }
  }

  static dynamic getAlertNumbers(String tid) async {
    var response = await HttpBase.getRequestWithHeader(
        isFullRoute: false,
        {},
        "pdon/alert/get_alert_number?issuing_app_id=${pluginConfig.appInfo.appId}&tid=$tid");
    if (response == 'failed' || response == null) {
      return false;
    } else {
      if (response['status'] != 'success') {
        return false;
      }

      return (response['data'] as List)
          .map((e) => AlertNumberModel.fromJson(e))
          .toList();
    }
  }

  static dynamic removeAlertNumbers(String id, String tid, String phone) async {
    var response = await HttpBase.postRequest(
        {"id": id, "tid": tid, "phone": phone}, 'pdon/alert/remove_number',
        returnErrorMsg: true, isFullRoute: false);

    if (response == 'failed' || response == null) {
      final error = BankBoxException(
          code: kUpdateUserError, message: 'User account update failed');
      pluginConfig.onError(error);
      return false;
    } else {
      return response['status'] == 'success';
    }
  }

  static Future<dynamic> getFliers({required String accountNumber}) async {
    final payload = {"account_number": "9441742671"};
    var response = await HttpBase.postRequest(payload, "generate_flier",
        isFullRoute: false);
    if (response == 'failed' ||
        response == null ||
        response['status'] != 'success') {
      final error = BankBoxException(
          code: kDeviceLabelUpdateError,
          message: 'Could not fetch flier, please try again');
      pluginConfig.onError(error);
      return false;
    }
    return response['data'];
  }

  static Future<dynamic> sendFlierViaEmail(
      {required String file,
      required String email,
      required String issueingId}) async {
    final payload = {"file": file, "email": email, "issuingAppId": issueingId};

    var response = await HttpBase.postRequest(payload, "pdon/send_flier",
        isFullRoute: false);
    if (response == 'failed' ||
        response == null ||
        response['status'] != 'success') {
      final error = BankBoxException(
          code: kDeviceLabelUpdateError,
          message: 'unable to send flier via email... please try again.');
      pluginConfig.onError(error);
      return false;
    }
    return response['data'];
  }

  static Future<dynamic> getPickupLocation() async {
    var response = await HttpBase.getRequestWithHeader({},
        "pdon/get/delivery_locations?issuingAppId=df7a6555f64c5eb3ec43f15c3d5c644f",
        isFullRoute: false);
    if (response == 'failed' ||
        response == null ||
        response['status'] != 'success') {
      final error = BankBoxException(
          code: kDeviceLabelUpdateError,
          message: 'unable to send flier via email... please try again.');
      pluginConfig.onError(error);
      return false;
    }
    if (response['status'] == "success") {
      return (response['data'] as List)
          .map((e) => PickupLocationModel.fromJson(e))
          .toList();
    } else {
      return false;
    }
  }
}
