import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:gap/gap.dart';
import 'package:mobile_pos/src/helpers/global_variables.dart';
import 'package:mobile_pos/src/helpers/helper_functions.dart';
import 'package:mobile_pos/src/models/alert_number_model.dart';
import 'package:mobile_pos/src/network/api_requests.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_bottomsheet.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_button_loader.dart';
import 'package:mobile_pos/src/styles/ravenpay_app_colors.dart';
import 'package:mobile_pos/src/styles/ravenpay_textstyles.dart';
import 'package:mobile_pos/src/views/terminal/alert_number/delete_alert_number_sheet.dart';
import 'package:mobile_pos/src/views/terminal/alert_number/edit_alert_number_sheet.dart';
import 'package:mobile_pos/src/views/terminal/widget/termianl_settings.dart';

class AlertNumberItem extends StatefulWidget {
  final AlertNumberModel model;
  final Function onDeleteDone;
  final Function onRefresh;
  final bool showDivider;
  const AlertNumberItem(
      {super.key,
      required this.model,
      required this.onRefresh,
      this.showDivider = true,
      required this.onDeleteDone});

  @override
  State<AlertNumberItem> createState() => _AlertNumberItemState();
}

class _AlertNumberItemState extends State<AlertNumberItem> {
  bool isBusy = false;
  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        Container(
          padding: const EdgeInsets.symmetric(vertical: 16),
          child: Row(
            children: [
              Container(
                  alignment: Alignment.center,
                  height: 32,
                  width: 32,
                  decoration: BoxDecoration(
                      shape: BoxShape.circle,
                      border: Border.all(
                          color: pluginTheme.primaryColor.withOpacity(0.5)),
                      color: pluginTheme.primaryColor.withOpacity(0.1)),
                  child: Text(
                    "${widget.model.fname[0]}${widget.model.lname[0]}"
                        .toUpperCase(),
                    style: bodyText.copyWith(
                        fontSize: 12, fontWeight: FontWeight.w500),
                  )),
              const Gap(12),
              Expanded(
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Text(
                        "${widget.model.fname} ${widget.model.lname}"
                            .capitalize(),
                        style: bodyText.copyWith(
                            fontSize: 14,
                            color: pluginTheme.isLightTheme
                                ? Colors.black
                                : Colors.white)),
                    const Gap(4),
                    Row(
                      children: [
                        Text(widget.model.phone,
                            style:
                                subtitle.copyWith(fontWeight: FontWeight.w400)),
                        Container(
                          height: 4,
                          width: 4,
                          margin: const EdgeInsets.symmetric(horizontal: 8),
                          decoration: BoxDecoration(
                              shape: BoxShape.circle,
                              color: AppColors.ravenPayGrey),
                        ),
                        Container(
                          padding: const EdgeInsets.symmetric(
                              horizontal: 8, vertical: 6),
                          decoration: BoxDecoration(
                              color: AppColors.ravenPayGrey.withOpacity(0.1),
                              borderRadius: BorderRadius.circular(12)),
                          child: Row(
                            children: [
                              if (widget.model.whatsappEnabled == 1) ...[
                                Image.asset(
                                  loadAsset("whatsapp_icon_outlined.png"),
                                  height: 12,
                                ),
                              ],
                              if (widget.model.whatsappEnabled == 1 &&
                                  widget.model.emailEnabled == 1) ...[
                                const Gap(8),
                              ],
                              if (widget.model.emailEnabled == 1) ...[
                                Image.asset(
                                  loadAsset("message_icon.png"),
                                  height: 11,
                                ),
                              ]
                            ],
                          ),
                        )
                      ],
                    )
                  ],
                ),
              ),
              const Gap(12),
              if (isBusy) ...[
                RavenButtonLoader(
                  size: 14,
                  color: pluginConfig.theme!.primaryColor,
                ),
              ] else ...[
                GestureDetector(
                  child: Row(
                    children: [
                      GestureDetector(
                        onTap: () async {
                          await showRavenPayBottomSheet(context,
                              child: EditAlertSheet(
                                model: widget.model,
                              ));
                          widget.onRefresh();
                        },
                        child: Image.asset(
                          loadAsset("edit_icon_new.png"),
                          height: 20,
                        ),
                      ),
                      const Gap(12),
                      GestureDetector(
                        onTap: () {
                          showRavenPayBottomSheet(context,
                              child: DeleteAlertNumberSheet(
                            onDelete: () {
                              removeAlertNumbers();
                            },
                          ));
                        },
                        child: Image.asset(
                          loadAsset("delete_icon.png"),
                          height: 20,
                        ),
                      ),
                    ],
                  ),
                )
              ]
            ],
          ),
        ),
        if (widget.showDivider) ...[
          const BankBoxDivider(
            height: 4,
          )
        ] else ...[
          const Gap(4)
        ]
      ],
    );
  }

  void removeAlertNumbers() async {
    isBusy = true;
    setState(() {});
    if (await ApiRequests.removeAlertNumbers(
        widget.model.id.toString(), widget.model.tid, widget.model.phone)) {
      widget.onDeleteDone();
    } else {
      isBusy = false;
      setState(() {});
    }
  }
}
