// ignore_for_file: use_build_context_synchronously

import 'package:flutter/material.dart';
import 'package:gap/gap.dart';
import 'package:mobile_pos/src/helpers/global_variables.dart';
import 'package:mobile_pos/src/helpers/helper_functions.dart';
import 'package:mobile_pos/src/models/alert_number_model.dart';
import 'package:mobile_pos/src/network/api_requests.dart';
import 'package:mobile_pos/src/shared_widgets/progress_dialog.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_bottomsheet_capsule.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_button.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_textfeild.dart';
import 'package:mobile_pos/src/styles/ravenpay_textstyles.dart';
import 'package:mobile_pos/src/views/terminal/view_terminal_details.dart';

class EditAlertSheet extends StatefulWidget {
  final AlertNumberModel model;

  const EditAlertSheet({super.key, required this.model});

  @override
  State<EditAlertSheet> createState() => _EditAlertSheetState();
}

class _EditAlertSheetState extends State<EditAlertSheet> {
  bool isBusy = false;
  Map selectedAlertType = {};
  @override
  void initState() {
    super.initState();
    firstnameController.text = widget.model.fname;
    lastnameController.text = widget.model.lname;
    emailController.text = widget.model.contactEmail ?? "";
    phoneController.text = widget.model.phone;
    if (widget.model.emailEnabled == 1) {
      selectedAlertType[AlertType.email] = AlertType.email;
    }
    if (widget.model.whatsappEnabled == 1) {
      selectedAlertType[AlertType.whatsapp] = AlertType.whatsapp;
    }
  }

  final firstnameController = TextEditingController();
  final lastnameController = TextEditingController();
  final phoneController = TextEditingController();
  final emailController = TextEditingController();

  final GlobalKey<FormState> formKey = GlobalKey<FormState>();
  @override
  Widget build(BuildContext context) {
    Size size = MediaQuery.of(context).size;
    return Container(
      padding: EdgeInsets.symmetric(horizontal: kHorizontalScreenPadding) +
          EdgeInsets.only(bottom: MediaQuery.of(context).viewInsets.bottom),
      child: Form(
        key: formKey,
        child: SingleChildScrollView(
          child: Column(
            mainAxisSize: MainAxisSize.min,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              const Gap(16),
              const RavenPayBottomSheetCapsule(),
              const Gap(24),
              Row(
                children: [
                  Expanded(
                    child: Text(
                      "Edit Alert Phone number",
                      style: headling1.copyWith(fontSize: 20),
                    ),
                  ),
                  GestureDetector(
                    onTap: () {
                      Navigator.pop(context);
                    },
                    child: const RoundedItemContainer(asset: "cancel_icon.png"),
                  )
                ],
              ),
              const Gap(4),
              Text(
                  "Ensure this phone number is connected to Whatsapp and is active.",
                  style: subtitle2.copyWith(fontSize: 14)),
              const Gap(24),
              RavenPayTextField(
                hintText: "Firstname",
                onChanged: (val) {
                  setState(() {});
                },
                controller: firstnameController,
                labelText: "Enter your Firstname",
                validator: (data) => validateInput(
                    minCount: 3, data, errorMessage: 'invalid firstname'),
                filledColor:
                    pluginTheme.isLightTheme ? const Color(0xffF7F8F7) : null,
              ),
              const Gap(20),
              RavenPayTextField(
                onChanged: (val) {
                  setState(() {});
                },
                validator: (data) => validateInput(
                    minCount: 3, data, errorMessage: 'invalid lastname'),
                hintText: "lastname",
                controller: lastnameController,
                labelText: "Enter your Lastname",
                filledColor:
                    pluginTheme.isLightTheme ? const Color(0xffF7F8F7) : null,
              ),
              const Gap(20),
              RavenPayTextField(
                counterText: '',
                maxLength: 11,
                validator: (data) => validateInput(
                    minCount: 10, data, errorMessage: 'invalid mobile number'),
                inputType: TextInputType.phone,
                controller: phoneController,
                isPhone: true,
                onChanged: (val) {
                  setState(() {});
                },
                labelText: "Phone Number",
              ),
              const Gap(20),
              RavenPayTextField(
                onChanged: (val) {
                  setState(() {});
                },
                validator: (data) => validateEmail(
                    minCount: 3, data, errorMessage: 'invalid email'),
                hintText: "email address",
                controller: emailController,
                labelText: "Enter your Email Address",
                filledColor:
                    pluginTheme.isLightTheme ? const Color(0xffF7F8F7) : null,
              ),
              DeliveryMethod(
                onChange: (val) {
                  if (selectedAlertType.containsKey(val)) {
                    selectedAlertType.remove(val);
                    setState(() {});
                    return;
                  }
                  selectedAlertType.putIfAbsent(val, () => val);
                  setState(() {});
                },
                selectedTypes: selectedAlertType,
              ),
              Gap(size.height * 0.08),
              RavenPayButton(
                  busy: isBusy,
                  buttonText: "Proceed",
                  onPressed: () async {
                    editAlertNumber();
                  }),
              const Gap(30),
            ],
          ),
        ),
      ),
    );
  }

  void editAlertNumber() async {
    if (!formKey.currentState!.validate()) {
      return;
    }

    String refinedPhone = '';
    if (phoneController.text.length == 10) {
      refinedPhone = '0${phoneController.text}';
    } else {
      refinedPhone = phoneController.text;
    }
    isBusy = true;
    setState(() {});
    showDialog(
        barrierDismissible: false,
        context: context,
        builder: (BuildContext context) => const ProgressDialog(
              status: 'Processing...',
            ));
    var response = await ApiRequests.updateAlertNumber(
        id: widget.model.id.toString(),
        phoneNumber: refinedPhone,
        tid: widget.model.tid,
        serial: widget.model.serial,
        firstname: firstnameController.text,
        email: emailController.text,
        whatsappEnabled: selectedAlertType.containsKey(AlertType.whatsapp),
        emailEnabled: selectedAlertType.containsKey(AlertType.email),
        lastname: lastnameController.text);
    Navigator.pop(context);
    isBusy = false;
    setState(() {});
    if (response['status'] != 'success') {
      Navigator.pop(context);
      showSnack(context, response['message']);
      return;
    }

    if (response['status'] == 'success') {
      showSnack(context, "alert number added successfully");
      Navigator.pop(context);
    }
  }
}

class DeliveryMethod extends StatelessWidget {
  final Function(AlertType) onChange;
  final Map selectedTypes;
  const DeliveryMethod(
      {super.key, required this.selectedTypes, required this.onChange});

  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        const Gap(20),
        Text("Delivery Method",
            style: subtitle.copyWith(
                fontSize: 14,
                color: pluginTheme.isLightTheme ? Colors.black : null,
                fontWeight: FontWeight.w500)),
        FittedBox(
          fit: BoxFit.scaleDown,
          child: Row(
            children: [
              item(
                  onChange: onChange,
                  name: AlertType.whatsapp,
                  selected: selectedTypes.containsKey(AlertType.whatsapp)),
              const Gap(16),
              item(
                  onChange: onChange,
                  name: AlertType.email,
                  selected: selectedTypes.containsKey(AlertType.email)),
              const Gap(16),
              item(
                  onChange: onChange,
                  name: AlertType.sms,
                  enabled: false,
                  selected: selectedTypes.containsKey(AlertType.sms)),
            ],
          ),
        )
      ],
    );
  }

  Widget item(
      {required AlertType name,
      required Function(AlertType) onChange,
      bool selected = false,
      bool enabled = true}) {
    return InkWell(
      highlightColor: Colors.transparent,
      splashColor: Colors.transparent,
      onTap: () {
        onChange(name);
      },
      child: Row(
        children: [
          SizedBox(
              width: 20,
              child: Checkbox(
                  value: enabled ? (selected) : false,
                  onChanged: (val) {
                    onChange(name);
                  })),
          const Gap(8),
          Text(name.name.toString().capitalize(),
              style: subtitle.copyWith(
                  fontWeight: FontWeight.w500,
                  color: subtitle.color!.withOpacity(enabled ? 1 : 0.4)))
        ],
      ),
    );
  }
}

enum AlertType { email, whatsapp, sms }
