import 'package:flutter/material.dart';
import 'package:gap/gap.dart';
import 'package:mobile_pos/src/helpers/global_variables.dart';
import 'package:mobile_pos/src/models/alert_number_model.dart';
import 'package:mobile_pos/src/network/api_requests.dart';
import 'package:mobile_pos/src/shared_widgets/raven_pay_fab.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_bottomsheet.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_button_loader.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_close_button.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_scaffold.dart';
import 'package:mobile_pos/src/styles/ravenpay_textstyles.dart';
import 'package:mobile_pos/src/views/terminal/alert_number/add_alert_number_sheet.dart';
import 'package:mobile_pos/src/views/terminal/alert_number/alert_number_item.dart';

class ViewAllAlertNumbers extends StatefulWidget {
  final String serial;
  final String tid;
  const ViewAllAlertNumbers(
      {super.key, required this.serial, required this.tid});

  @override
  State<ViewAllAlertNumbers> createState() => _ViewAllAlertNumbersState();
}

class _ViewAllAlertNumbersState extends State<ViewAllAlertNumbers> {
  bool loading = true;
  List<AlertNumberModel> allAlertNumbers = [];

  void fetchAlertNumbers({bool showLoading = true}) async {
    loading = showLoading;
    var response = await ApiRequests.getAlertNumbers(widget.tid);
    if (response != false) {
      allAlertNumbers = response;
    }
    loading = false;
    setState(() {});
  }

  @override
  void initState() {
    super.initState();
    fetchAlertNumbers();
  }

  @override
  Widget build(BuildContext context) {
    return RavenPayScaffold(
        fab: loading
            ? const SizedBox()
            : RavenPayFab(
                buttonColor: pluginTheme.primaryColor,
                onPressed: () async {
                  await showRavenPayBottomSheet(context,
                      child: AddAlertSheet(
                          serial: widget.serial, tid: widget.tid));
                  fetchAlertNumbers();
                },
                widget: Row(mainAxisSize: MainAxisSize.min, children: [
                  Icon(Icons.add, color: pluginTheme.onPrimary),
                  const Gap(4),
                  Text(
                    "Add New Alert",
                    style: subtitle.copyWith(
                        fontSize: 14, color: pluginTheme.onPrimary),
                  ),
                ]),
              ),
        body: Column(children: [
          Padding(
              padding:
                  EdgeInsets.symmetric(horizontal: kHorizontalScreenPadding),
              child: Column(
                children: [
                  const Gap(24),
                  const RavenPayCloseButton(
                    text: "",
                    isArrow: true,
                  ),
                  const Gap(24),
                  Align(
                    alignment: Alignment.centerLeft,
                    child: Text(
                      "Alert Numbers",
                      style: headling2.copyWith(fontSize: 20),
                    ),
                  ),
                ],
              )),
          const Gap(8),
          Expanded(
            child: loading == true
                ? Center(
                    child: RavenButtonLoader(
                      size: 18,
                      color: pluginConfig.theme!.primaryColor,
                    ),
                  )
                : allAlertNumbers.isEmpty
                    ? Center(
                        child: Column(
                          crossAxisAlignment: CrossAxisAlignment.center,
                          mainAxisSize: MainAxisSize.min,
                          children: [
                            const Gap(20),
                            Text(
                              "No alert number found",
                              style: headling1.copyWith(fontSize: 18),
                              textAlign: TextAlign.center,
                            ),
                            const Gap(8),
                            Padding(
                              padding:
                                  const EdgeInsets.symmetric(horizontal: 32),
                              child: Text(
                                "You do not have any alert number attached to your account",
                                style: subtitle2.copyWith(fontSize: 14),
                                textAlign: TextAlign.center,
                              ),
                            ),
                            Gap(MediaQuery.of(context).size.height / 8),
                          ],
                        ),
                      )
                    : RefreshIndicator(
                        color: pluginTheme.primaryColor,
                        onRefresh: () async {
                          await Future.delayed(const Duration(seconds: 2));
                          fetchAlertNumbers(showLoading: false);
                        },
                        child: Container(
                          margin: const EdgeInsets.symmetric(horizontal: 16),
                          decoration: BoxDecoration(
                              boxShadow: pluginTheme.isLightTheme
                                  ? [
                                      BoxShadow(
                                          color: Colors.grey.withOpacity(0.2),
                                          spreadRadius: 0,
                                          blurRadius: 44)
                                    ]
                                  : [],
                              color: Colors.white.withOpacity(
                                  pluginTheme.isLightTheme ? 1 : 0.08),
                              borderRadius: BorderRadius.circular(12)),
                          child: ListView.builder(
                              shrinkWrap: true,
                              padding: EdgeInsets.symmetric(
                                      horizontal: kHorizontalScreenPadding) +
                                  const EdgeInsets.only(top: 4),
                              itemCount: allAlertNumbers.length,
                              itemBuilder: (context, index) {
                                return AlertNumberItem(
                                  onRefresh: () {
                                    fetchAlertNumbers(showLoading: false);
                                  },
                                  onDeleteDone: () {
                                    fetchAlertNumbers(showLoading: true);
                                    setState(() {});
                                  },
                                  showDivider:
                                      index != (allAlertNumbers.length - 1),
                                  model: allAlertNumbers[index],
                                );
                              }),
                        ),
                      ),
          )
        ]));
  }
}
