// ignore_for_file: use_build_context_synchronously

import 'package:flutter/material.dart';
import 'package:gap/gap.dart';
import 'package:mobile_pos/src/helpers/global_variables.dart';
import 'package:mobile_pos/src/helpers/helper_functions.dart';
import 'package:mobile_pos/src/network/api_requests.dart';
import 'package:mobile_pos/src/shared_widgets/progress_dialog.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_bottomsheet_capsule.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_button.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_textfeild.dart';
import 'package:mobile_pos/src/styles/ravenpay_textstyles.dart';
import 'package:mobile_pos/src/views/terminal/view_terminal_details.dart';
import 'package:share_plus/share_plus.dart';

class PaymentFlierSheet extends StatefulWidget {
  final String flierLink;

  const PaymentFlierSheet({super.key, required this.flierLink});

  @override
  State<PaymentFlierSheet> createState() => _PaymentFlierSheetState();
}

class _PaymentFlierSheetState extends State<PaymentFlierSheet> {
  final emailController = TextEditingController();
  @override
  void initState() {
    super.initState();
  }

  bool? emailSelected;

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: EdgeInsets.symmetric(horizontal: kHorizontalScreenPadding) +
          EdgeInsets.only(bottom: MediaQuery.of(context).viewInsets.bottom),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          const Gap(16),
          const RavenPayBottomSheetCapsule(),
          const Gap(24),
          Row(
            children: [
              Expanded(
                child: Text(
                  "Flier Generated",
                  style: headling1.copyWith(fontSize: 20),
                ),
              ),
              GestureDetector(
                onTap: () {
                  Navigator.pop(context);
                },
                child: const RoundedItemContainer(asset: "cancel_icon.png"),
              )
            ],
          ),
          const Gap(24),
          Container(
            padding: const EdgeInsets.symmetric(horizontal: 12, vertical: 16),
            decoration: BoxDecoration(
                color: Colors.white
                    .withOpacity(pluginTheme.isLightTheme ? 1 : 0.04),
                borderRadius: BorderRadius.circular(12)),
            child: Column(
              children: [
                Container(
                  decoration: BoxDecoration(
                      color: pluginTheme.primaryColor.withOpacity(0.08),
                      borderRadius: BorderRadius.circular(12)),
                  padding:
                      const EdgeInsets.symmetric(horizontal: 12, vertical: 16),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text("Account flier link",
                          style: headling2.copyWith(
                              fontSize: 14,
                              fontWeight: FontWeight.w200,
                              fontFamily: 'br_regular')),
                      const Gap(4),
                      Row(
                        children: [
                          Expanded(
                            child: Text(widget.flierLink,
                                maxLines: 1,
                                overflow: TextOverflow.ellipsis,
                                style: headling1.copyWith(
                                    fontSize: 14,
                                    fontWeight: FontWeight.w200,
                                    fontFamily: 'br_regular')),
                          ),
                          const Gap(12),
                          GestureDetector(
                            onTap: () {
                              Navigator.pop(context);
                              copyToClipBoard(context,
                                  data: widget.flierLink,
                                  msg: "${widget.flierLink} copied");
                            },
                            child: Icon(
                              Icons.copy,
                              color: pluginTheme.primaryColor,
                              size: 14,
                            ),
                          )
                        ],
                      )
                    ],
                  ),
                ),
                const Gap(16),
                Row(
                  children: [
                    item(
                        asset: "email_new_icon",
                        title: "Share to email",
                        onTap: () {
                          emailSelected = true;
                          setState(() {});
                        }),
                    item(
                        asset: "share_new_icon",
                        title: "Share flier link",
                        onTap: () {
                          emailSelected = false;
                          setState(() {});
                          Share.share(widget.flierLink);
                        })
                  ],
                ),
                if (emailSelected != null && emailSelected!) ...[
                  const Gap(24),
                  RavenPayTextField(
                    controller: emailController,
                    labelText: "Email Address",
                    hintText: "john@mail.com",
                  ),
                  const Gap(24),
                  RavenPayButton(
                    buttonText: "Share",
                    onPressed: () {
                      seneFlierViaEmail();
                    },
                  ),
                  const Gap(12),
                ]
              ],
            ),
          ),
          const Gap(54),
        ],
      ),
    );
  }

  seneFlierViaEmail() async {
    if (!emailController.text.contains("@")) {
      showSnack(context, "invalid email provided");
      return;
    }
    showDialog(
        barrierDismissible: false,
        context: context,
        builder: (BuildContext context) => const ProgressDialog(
              status: 'please wait...',
            ));
    List<String> data = widget.flierLink.split("/");
    String file = data[data.length - 1];
    var response = await ApiRequests.sendFlierViaEmail(
        file: file,
        email: emailController.text,
        issueingId: pluginConfig.appInfo.appId);
    Navigator.pop(context);
    if (response != false) {
      Navigator.pop(context);
      showSnack(context, "flier sent");
    }
  }

  Widget item(
      {required String title, required String asset, required Function onTap}) {
    return Expanded(
      child: GestureDetector(
        onTap: () => onTap(),
        child: FittedBox(
          fit: BoxFit.scaleDown,
          child: Row(
            children: [
              RoundedItemContainer(asset: "$asset.png"),
              const Gap(8),
              Text(
                title,
                style: subtitle.copyWith(fontWeight: FontWeight.w400),
              )
            ],
          ),
        ),
      ),
    );
  }
}
