import 'package:flutter/material.dart';
import 'package:gap/gap.dart';
import 'package:mobile_pos/src/helpers/extensions.dart';
import 'package:mobile_pos/src/helpers/global_variables.dart';
import 'package:mobile_pos/src/models/pick_up_location.dart';
import 'package:mobile_pos/src/network/api_requests.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_bottomsheet_capsule.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_button_loader.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_textfeild.dart';
import 'package:mobile_pos/src/styles/ravenpay_app_colors.dart';
import 'package:mobile_pos/src/styles/ravenpay_textstyles.dart';

class PickupLocationSheet extends StatefulWidget {
  const PickupLocationSheet({super.key});

  @override
  PickupLocationSheetState createState() => PickupLocationSheetState();
}

class PickupLocationSheetState extends State<PickupLocationSheet> {
  bool isLoading = true;
  List<PickupLocationModel> pickupLocations = [];
  final searchController = TextEditingController();

  @override
  void initState() {
    fetchPickUpLocations();
    super.initState();
  }

  fetchPickUpLocations() async {
    isLoading = true;
    setState(() {});
    var response = await ApiRequests.getPickupLocation();
    if (response == false) {
      return;
    }
    isLoading = false;
    setState(() {});
    pickupLocations = response;
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: const EdgeInsets.only(top: 16, left: 16, right: 16, bottom: 10),
      height: MediaQuery.of(context).size.height * .92,
      child: Column(
        children: [
          const RavenPayBottomSheetCapsule(),
          const Gap(8),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              Text(
                'Pickup Location',
                style: headling1.copyWith(
                    fontSize: 18,
                    color: pluginTheme.primaryColor,
                    fontWeight: FontWeight.w700),
              ),
              InkWell(
                  onTap: () {
                    Navigator.pop(context);
                  },
                  child: Icon(Icons.close, color: pluginTheme.primaryColor))
            ],
          ),
          RavenPayTextField(
            controller: searchController,
            contentPadding:
                const EdgeInsets.symmetric(horizontal: 16, vertical: 16),
            hintText: 'Search Pickup Location',
            onChanged: (value) {
              setState(() {});
            },
          ),
          const Gap(12),
          isLoading
              ? Expanded(
                  child: Center(
                    child: RavenButtonLoader(
                      size: 18,
                      color: pluginConfig.theme!.primaryColor,
                    ),
                  ),
                )
              : pickupLocations.isEmpty
                  ? Expanded(
                      child: Center(
                        child: Text(
                          "No available Pickup Location",
                          style: subtitle,
                        ),
                      ),
                    )
                  : Expanded(
                      child: ListView.builder(
                          itemCount: item().length,
                          itemBuilder: (context, index) {
                            return ItemWidget(model: item()[index]);
                          }),
                    )
        ],
      ),
    );
  }

  List<PickupLocationModel> item() {
    return pickupLocations
        .where((element) =>
            element.address
                .toUpperCase()
                .contains(searchController.text.toUpperCase()) ||
            element.lga
                .toUpperCase()
                .contains(searchController.text.toUpperCase()) ||
            element.state
                .toUpperCase()
                .contains(searchController.text.toUpperCase()))
        .toList();
  }
}

class ItemWidget extends StatelessWidget {
  final PickupLocationModel model;
  const ItemWidget({super.key, required this.model});

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        Navigator.pop(context, model);
      },
      child: Container(
          margin: const EdgeInsets.only(bottom: 12),
          width: double.infinity,
          height: 108,
          decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(12),
              color: pluginTheme.isLightTheme
                  ? AppColors.lightGrey3.withOpacity(0.4)
                  : AppColors.darkModeBorder.withOpacity(0.4),
              border: Border.all(
                  color: pluginTheme.isLightTheme
                      ? AppColors.lightGrey3
                      : AppColors.darkModeBorder)),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Padding(
                padding: const EdgeInsets.symmetric(horizontal: 12) +
                    const EdgeInsets.only(top: 12, bottom: 4),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Text(
                        "${model.address.capitalize()}, ${model.lga.capitalize()}. ${model.state.capitalize()}",
                        maxLines: 1,
                        style: headling2.copyWith(fontSize: 14)),
                    const Gap(4),
                    Text("08:00AM - 5:00PM",
                        style: subtitle.copyWith(fontSize: 12)),
                  ],
                ),
              ),
              Expanded(
                child: Container(
                    padding: const EdgeInsets.symmetric(horizontal: 12),
                    width: double.infinity,
                    height: double.infinity,
                    decoration: BoxDecoration(
                      color: pluginTheme.getFGColor,
                      borderRadius: BorderRadius.circular(12),
                    ),
                    child: Align(
                      alignment: Alignment.centerLeft,
                      child: FittedBox(
                        fit: BoxFit.scaleDown,
                        child: Row(
                          children: [
                            ...model.pickUpDays
                                .split(",")
                                .map((e) => item(days: e))
                          ],
                        ),
                      ),
                    )),
              )
            ],
          )),
    );
  }

  Widget item({required String days}) {
    return days.isEmpty
        ? const SizedBox(
            width: 1,
            height: 1,
          )
        : Container(
            margin: const EdgeInsets.symmetric(horizontal: 4),
            padding: const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
            decoration: BoxDecoration(
                color: pluginTheme.isLightTheme
                    ? AppColors.lightGrey3.withOpacity(0.4)
                    : AppColors.darkModeBorder.withOpacity(0.4),
                borderRadius: BorderRadius.circular(24)),
            child: Text(days, style: subtitle.copyWith(color: headling2.color)),
          );
  }
}
