// ignore_for_file: use_build_context_synchronously

import 'package:flutter/material.dart';
import 'package:gap/gap.dart';
import 'package:mobile_pos/src/helpers/global_variables.dart';
import 'package:mobile_pos/src/helpers/helper_functions.dart';
import 'package:mobile_pos/src/models/terminal_model.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_bottomsheet.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_bottomsheet_capsule.dart';
import 'package:mobile_pos/src/styles/ravenpay_app_colors.dart';
import 'package:mobile_pos/src/styles/ravenpay_textstyles.dart';
import 'package:mobile_pos/src/views/terminal/alert_number/view_alert_number.dart';
import 'package:mobile_pos/src/views/terminal/view_terminal_details.dart';
import 'package:mobile_pos/src/views/terminal/widget/disable_terminal_sheet.dart';
import 'package:mobile_pos/src/views/terminal/widget/update_terminal_sheet.dart';

class TerminalSettings extends StatefulWidget {
  final TerminalModel model;
  final Function onUpdated;
  const TerminalSettings(
      {super.key, required this.model, required this.onUpdated});

  @override
  State<TerminalSettings> createState() => _TerminalSettingsState();
}

class _TerminalSettingsState extends State<TerminalSettings> {
  @override
  Widget build(BuildContext context) {
    return Container(
      padding: EdgeInsets.symmetric(horizontal: kHorizontalScreenPadding),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          const Gap(16),
          const RavenPayBottomSheetCapsule(),
          const Gap(24),
          Row(
            children: [
              Expanded(
                child: Text(
                  "Terminal Settings",
                  style: headling1.copyWith(fontSize: 20),
                ),
              ),
              GestureDetector(
                onTap: () {
                  Navigator.pop(context);
                },
                child: const RoundedItemContainer(
                  padding: 5,
                  asset: "cancel_icon.png",
                  showContainerColor: false,
                ),
              )
            ],
          ),
          const Gap(20),
          Container(
            padding: const EdgeInsets.symmetric(horizontal: 12, vertical: 28),
            decoration: BoxDecoration(
                color: Colors.white
                    .withOpacity(pluginTheme.isLightTheme ? 1 : 0.02),
                borderRadius: BorderRadius.circular(12)),
            child: Column(
              children: [
                ItemWidget(
                  iconPadding: 8,
                  name: "Edit terminal label",
                  onTap: () {
                    Navigator.pop(context);
                    showRavenPayBottomSheet(context,
                        child: UpdateTerminalInfo(
                          model: widget.model,
                          onUpdated: widget.onUpdated,
                        ));
                  },
                  asset: "edit_icon2",
                ),
                const BankBoxDivider(
                  color: Colors.transparent,
                  height: 30,
                ),
                ItemWidget(
                  name: "Raise an issue or complaint",
                  onTap: () {
                    Navigator.pop(context);
                  },
                  asset: "info_icon",
                ),
                const BankBoxDivider(
                  color: Colors.transparent,
                  height: 30,
                ),
                ItemWidget(
                  name: "Generate payment flier",
                  onTap: () {
                    Navigator.pop(context, "REQUEST_FLIER");
                  },
                  asset: "payment_flyer",
                ),
                const BankBoxDivider(
                  color: Colors.transparent,
                  height: 30,
                ),
                ItemWidget(
                  name: "Alert Numbers",
                  onTap: () {
                    Navigator.pop(context);
                    pushRoute(
                        context,
                        ViewAllAlertNumbers(
                          tid: widget.model.tid ?? "",
                          serial: widget.model.serial ?? "",
                        ));
                  },
                  asset: "alert_icon",
                ),
                const BankBoxDivider(
                  height: 30,
                  color: Colors.transparent,
                ),
                ItemWidget(
                  name: "Disable this terminal",
                  onTap: () {
                    Navigator.pop(context);
                    showRavenPayBottomSheet(context,
                        child: DisableTerminalSheet(model: widget.model));
                  },
                  asset: "cancel_icon",
                  color: AppColors.ravenRed,
                ),
              ],
            ),
          ),
          const Gap(24)
        ],
      ),
    );
  }
}

class BankBoxDivider extends StatelessWidget {
  final double height;
  final Color? color;
  const BankBoxDivider({super.key, this.height = 0, this.color});

  @override
  Widget build(BuildContext context) {
    return Divider(
      height: height,
      color: color ??
          AppColors.ravenLightGrey
              .withOpacity(pluginTheme.isLightTheme ? 0.7 : 0.08),
    );
  }
}

class ItemWidget extends StatelessWidget {
  final String name;
  final String asset;
  final Function onTap;
  final Color? color;
  final double iconPadding;
  const ItemWidget(
      {super.key,
      required this.name,
      this.iconPadding = 7,
      this.color,
      required this.onTap,
      required this.asset});

  @override
  Widget build(BuildContext context) {
    return InkWell(
      splashColor: Colors.transparent,
      highlightColor: Colors.transparent,
      onTap: () => onTap(),
      child: Row(
        children: [
          Container(
            padding: EdgeInsets.all(iconPadding),
            height: 32,
            width: 32,
            decoration: BoxDecoration(
              color: pluginTheme.isLightTheme
                  ? Colors.grey.withOpacity(0.08)
                  : Colors.white.withOpacity(0.04),
              shape: BoxShape.circle,
            ),
            child: Image.asset(
              loadAsset("$asset.png"),
              color: color,
              height: 20,
            ),
          ),
          const Gap(16),
          Expanded(child: Text(name, style: subtitle.copyWith(fontSize: 14)))
        ],
      ),
    );
  }
}
