import 'package:flutter/material.dart';
import 'package:gap/gap.dart';
import 'package:mobile_pos/src/helpers/global_variables.dart';
import 'package:mobile_pos/src/helpers/helper_functions.dart';
import 'package:mobile_pos/src/models/terminal_model.dart';
import 'package:mobile_pos/src/styles/ravenpay_app_colors.dart';
import 'package:mobile_pos/src/styles/ravenpay_textstyles.dart';
import 'package:mobile_pos/src/views/terminal/terminal_status.dart';

class TerminalRequestItem extends StatelessWidget {
  final TerminalRequestModel model;

  const TerminalRequestItem({
    super.key,
    required this.model,
  });

  @override
  Widget build(BuildContext context) {
    return InkWell(
      splashColor: Colors.transparent,
      highlightColor: Colors.transparent,
      onTap: () {
        pushRoute(
            context,
            TerminalStatus(
              model: model,
            ));
      },
      child: Column(
        children: [
          const Gap(8),
          Row(
            children: [
              Container(
                height: 40,
                width: 40,
                decoration: BoxDecoration(
                    shape: BoxShape.circle,
                    color: AppColors.ravenLightGrey.withOpacity(0.3)),
                child: Icon(Icons.info_outline_rounded,
                    color: AppColors.ravenLightGrey),
              ),
              const Gap(16),
              Expanded(
                child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text('Terminal Request (x${model.quantity})',
                          style: headling2.copyWith(
                              color: pluginTheme.primaryColor, fontSize: 14)),
                      const Gap(4),
                      Text(
                        getTerminalStatus(model.status.toString()),
                        style: subtitle2.copyWith(
                            fontSize: 14, color: AppColors.ravenGreyLight1),
                      )
                    ]),
              )
            ],
          ),
          const Gap(8),
          const Divider()
        ],
      ),
    );
  }
}

String getTerminalStatus(String status) {
  if (status == "0") {
    return "Pending Approval";
  }
  if (status == "1") {
    return "BankBox Approved";
  }
  if (status == "2") {
    return "BankBox Dispatched";
  }
  if (status == "3") {
    return "BankBox Delivered";
  }
  return "Pending Approval";
}

class CardShadowViewLight extends StatelessWidget {
  final Widget child;
  final double? vertical;
  const CardShadowViewLight({super.key, required this.child, this.vertical});

  @override
  Widget build(BuildContext context) {
    return Container(
        margin: const EdgeInsets.symmetric(vertical: 10),
        padding: const EdgeInsets.symmetric(vertical: 4, horizontal: 4),
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.circular(12),
          boxShadow: [
            BoxShadow(
              color: Colors.grey.withOpacity(.07),
              blurRadius: 18,
              spreadRadius: 7,
            ),
          ],
        ),
        child: child);
  }
}
